﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using InteractiveData.eSignalDesktop.Trading;
using InteractiveData.eSignalDesktop.Trading.Attributes;
using InteractiveData.eSignalDesktop.Trading.Exceptions;
using Exception = System.Exception;

namespace CQGPlugin
{
    // Declare enumeration that will be used for our order types. 
    // See CSharpSamplePlugin.GetSupportedOrderTypes() for more details.
    internal enum MySupportedOrderTypes
    {
        Market,
        Limit,
        Stop,
        StopLimit,
        TrailingStop,
        MarketIfTouched,
        LimitIfTouched
    }

    public sealed class PluginConnectionSettings : ConnectionSettings
    {
    }

    public sealed class PluginOrderData : OrderData
    {
        public PluginOrderData(string Id, string Symbol) 
            : base(Id, Symbol)
        {
        }
    }

    // This attribute marks add-in implementation and sets values of some additional properties.
    [Plugin(
        SupportOfMMSShouldBeEnabled = true,
        AreOrderIDsPermanent = true,
        DataStreamingSupported = true)]

    [SupportedSymbolTypes(
        SymbolType.Future,
        SymbolType.Stock,
        SymbolType.Index,
        SymbolType.Forex,
        SymbolType.StockOption,
        SymbolType.FutureOption,
        SymbolType.IndexOption,
        SymbolType.MutualFund,
        SymbolType.MoneyMktFund,
        SymbolType.Bond,
        SymbolType.Spread,
        SymbolType.UserDefinedSpread)]

    [Dependency(HostComponents.EsignalDesktop, "11.8.0.0")]

    public sealed class Plugin : IPlugin
    {
        public void OnHostAttach(IPluginHost Host)
        {
            PluginHost = Host;

            PluginHost.SymbolSubscribed += OnSymbolSubscribed;
            PluginHost.SymbolUnsubscribed += OnSymbolUnsubscribed;
        }

        public void OnHostDetach()
        {
            PluginHost = null;
        }

        #region Connect/Disconnect

        public void Connect(ConnectionSettings ConnectionSettings)
        {
            PluginHost.SendConnectionState(ConnectionState.Connected);

            const String AccountName = "Demo USD Account";

            PluginHost.SendAccount(new AccountData(AccountName)
            {
                Balance = 100000
            });

            PluginHost.EndOfAllHistory();
        }

        public void Disconnect()
        {
            PluginHost.SendConnectionState(ConnectionState.Disconnected);
        }

        #endregion
        
        #region Orders

        public void PlaceOrder(OrderData order, IMethodRequest request)
        {
        }

        public void ReplaceOrder(OrderData order, IMethodRequest request)
        {
        }

        public void CancelOrder(String orderId, IMethodRequest request)
        {
        }

        public void UpdateControls(IOrderTicket orderTicket)
        {
        }

        #endregion

        public OrderTypeDeclaration[] GetSupportedOrderTypes()
        {
            return new[]
            {
                new OrderTypeDeclaration(StandardOrderType.Market, new OrderType(MySupportedOrderTypes.Market)),
                new OrderTypeDeclaration(StandardOrderType.Limit, new OrderType(MySupportedOrderTypes.Limit)),
                new OrderTypeDeclaration(StandardOrderType.Stop, "Stop Order", new OrderType(MySupportedOrderTypes.Stop)),
                new OrderTypeDeclaration(StandardOrderType.StopLimit, "Stop-Limit Order", new OrderType(MySupportedOrderTypes.StopLimit)),
                new OrderTypeDeclaration("Trailing Stop", new OrderType(MySupportedOrderTypes.TrailingStop))
            };
        }

        public string MapSymbolFrom(IInputSymbol symbol)
        {
            try
            {
                var searchResult = SymbolSearch.SymbolSearch.Search(symbol.ESignalSymbol);
                Trace.TraceInformation("Mapped {0} on {1}. Exchange: {2}. Description: {3}", symbol.Symbol, searchResult.Symbol, searchResult.Exchange, searchResult.Description);
                return searchResult.Symbol;
            }
            catch (InvalidOperationException)
            {
                Trace.TraceError("Can't find anything");
                return null;
            }
            catch (ArgumentOutOfRangeException)
            {
                Trace.TraceError("Not supported symbol type");
                return null;
            }
        }

        private void OnSymbolSubscribed(string symbolId)
        {
            try
            {
                var searchResult = SymbolSearch.SymbolSearch.SearchByCqgSymbol(symbolId);

                var symbolData = new SymbolData(symbolId)
                {
                    Description = searchResult.Description,
                    Exchange = searchResult.Exchange,
                    Type = searchResult.Type == "Future" ? SymbolType.Future : SymbolType.FutureOption,
                    Valid = true,

                    //temp stub
                    MinMovement = 1,
                    PriceScale = (int)(1 / searchResult.Tick.Size),
                    PointValue = searchResult.Tick.PointValue / searchResult.Tick.Size
                };

                PluginHost.SendSymbol(symbolId, symbolData);
            }
            catch (InvalidOperationException)
            {
                var symbolData = new SymbolData(symbolId){Valid = false};
                PluginHost.SendSymbol(symbolId, symbolData);
            }
        }

        private void OnSymbolUnsubscribed(string symbolId)
        {
        }

        public IConnectionSettingsPage CreateConnectionSettingsPage()
        {
            return null;
        }

        public ISymbolLookupPage CreateSymbolLookupPage()
        {
            return null;
        }

        #region Others

        private void OnSymbolQuoteSubscribed(String symbolId)
        {
        }

        private void OnSymbolQuoteUnsubscribed(String symbolId)
        {
        }

        private void OnSymbolMarketDepthSubscribed(String symbolId)
        {
        }

        private void OnSymbolMarketDepthUnsubscribed(String symbolId)
        {
        }

        #endregion


        private IPluginHost PluginHost { get; set; }
    }
}